def multi_frequency_fit(time, *params):
    """
    This function collects an array of times and an arbitrary (but equal) number of frequencies, amplitudes and phases.
    It adds all of the sine waves together and returns their sum for each observation time.
    Args:
        Time : np.ndarray
            Input times
        Params : List or tuple
            Input parameters in order of amplitude, frequency, and phase.
            Should take the form [[amplitude], [frequency], [phase]] or
            [amplitude, frequency, phase]
    Example usage:
        time = np.linspace(0,np.pi,101)
        params = [[1],[1],[1]]
        multi_frequency_fit(time, params)
    """
    
    # Flatten input array to ensure compatibility with scipy.optimize
    params = np.array(params).flatten()
    
    # collect the parameters
    amps=params[:int(len(params)/3)]
    freqs = params[int(len(params)/3):int(2*len(params)/3)]
    phases = params[int(2*len(params)/3):]

    # sum the sine waves
    nfreq = len(freqs)
    y = np.zeros_like(time)
    for j in np.arange(nfreq):
        y += amps[j] * np.sin(2*np.pi*freqs[j]*time+phases[j])

    # return the sum
    return y



def optimizer(time,mag,freqs):
    """
    This function sets up the variables to feed the curve_fit, runs that curve_fit, and collects the output.
    It returns the amplitudes, frequencies and phases as well as their uncertainties.
    """

    # prepare some empty lists to collect our amplitudes and phases
    amps = []
    phases = []

    # for every frequency, we need to initialize an amplitude and phase.
    # the initial values for amp and phase don't matter -- the optimizer will handle it
    for j in freqs:
        amps.append(0.001)
        phases.append(0.0)

    # run the curve_fit
    # p is the list of parameters, and pcov is the covariance matrix
    p,pcov = optimize.curve_fit(multi_frequency_fit, time, mag, p0=np.array([amps,freqs,phases]))

    # collect the results
    a_out = p[:int(len(p)/3)]
    freq_out = p[int(len(p)/3):int(2*len(p)/3)]
    phi_out = p[int(2*len(p)/3):]

    # technically some of the amplitudes could come out negative, if their phase is off by pi radians
    # that would be a problem for us later, so let's fix that
    for i in np.arange(len(a_out)):
        if a_out[i] < 0:
            a_out[i] = np.abs(a_out[i])
            phi_out[i] += np.pi

    # collect the uncertainties
    perr = np.sqrt(np.diag(pcov))
    a_err = perr[:int(len(perr)/3)]
    freq_err = perr[int(len(perr)/3):int(2*len(perr)/3)]
    phi_err = perr[int(2*len(perr)/3):]

    # return the parameters and the uncertainties
    return a_out, freq_out, phi_out, a_err, freq_err, phi_err
